/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : EEPROM_Platformservice.c
   Author             : Rajesh
   Version            : V0.0.0.1
   Origination Date   : 04/03/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the EEPROM platform service initialization

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             04/03/2022           Rajesh

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../Inc/Common.h"
#include "../user_modify_files/EE_User_define.h"
/* USER CODE END Includes */

/* USER CODE BEGIN Global Variables */
#if (SDK_SERVICE_EEPROM == PS_ENABLE)
extern pvt_variable pv;

/* USER CODE BEGIN Global Variables */

//uint8_t Shadow_addr[EEPROM_SIZE];
uint8_t *Shadow_addr = (uint8_t *)EEPROM_SHADOW_ADDRESS;

/*!
 * EEPROM Placeholder
 */
EE_PLACEHOLDER      *EE_PLACEHOLDER_data =
		(EE_PLACEHOLDER *)(EEPROM_SHADOW_ADDRESS + END_EEPROM);

/* USER CODE END Global Variables */

/** @brief Initialize EEPROM Placeholder
 *
 *  This function takes the default configuration from the USER
 *  EEPROM config header file and store the USER data to the
 *  EEPROM Placeholder.
 *
 *  @param  None
 *
 *  @return None
 *
 */
void Initialize_EEPlaceHolder()
{

	uint16_t index;

	uint8_t *USER_Shadow_addr = (&Shadow_addr[0] + MAGIC_NUMBER_SIZE);

	/* Copy the USER data to the place holder */
	for (index = 0; index < (EE_USER_END - (EE_USER_START+1)); index ++)
	{
		EE_PLACEHOLDER_data[index].ID = EE_user_elements[index].ID;
		EE_PLACEHOLDER_data[index].EE_info = &EE_user_elements[index];
		EE_PLACEHOLDER_data[index].Shadow_EE_data_addr = (uint8_t *)USER_Shadow_addr;
		EE_PLACEHOLDER_data[index].Update_data = FALSE;
		EE_PLACEHOLDER_data[index].EE_status   = INVALID_EE_DATA;

		/* Find out the next place holder */
		USER_Shadow_addr = USER_Shadow_addr + \
		   ((EE_user_elements[index].Size + 1) * (EE_user_elements[index].Redundancy + 1));
	}
}

/** @brief Get data from EEPROM Shadow location
 *
 *  This function get the data from the EEPROM Shadow location.
 *  Calculate the CRC for the data if CRC presents and return
 *  the proper data. If the CRC is not valid for all the location
 *  then returns the invalid data.
 *
 *
 *  @param  eeplace_holder[IN] : EEPROM Place holder ID
 *          data[IN]           : User buffer
 *          size[IN]           : User buffer size
 *
 *  @return  x : Valid value
 *          -1 : InValid value
 */
int8_t GET_Data_From_EE_Shadow(uint8_t eeplace_holder, uint8_t *data, uint8_t size)
{
	uint8_t *addr;

	if (NULL != data)
		return INVALID_EE_DATA;

	/* Get the location of the EEPROM from the placeholder structure */
	addr = EE_PLACEHOLDER_data[eeplace_holder].Shadow_EE_data_addr;

	/* Check for the buffer size */
    if(size < EE_PLACEHOLDER_data[eeplace_holder].EE_info->Size)
        size = EE_PLACEHOLDER_data[eeplace_holder].EE_info->Size;

    /* Copy the data to the user buffer */
    memcpy(data, addr, size);

    return VALID_EE_DATA;
}

/** @brief Copy the EEPROM data to Shadow addr
 *
 *  This function reads the data from the EEPROM and store it in
 *  shadow address location.
 *
 *
 *  @param   None
 *
 *  @return  None
 *
 */
void CopyEEPROMDatatoShadowAddr()
{
	/*
	 * Read all the data from the EEPROM and
	 * Copy to the Shadow address
	 */
	TakeSPIBusLock();
    EEPROM_SPI_ReadBuffer(&Shadow_addr[0], 0x00, (uint16_t)(EEPROM_SIZE + MAGIC_NUMBER_SIZE));
    GiveSPIBusLock();
}


/** @brief Initialize the EEPROM Place holder
 *
 *  This function does the following.
 *         -- Initialize the EEPROM PlaceHolder fields
 *         -- Link the EEPROM placeholder with the EEPROM shadow_addr
 *         -- Copy the data the EEPROM if the data is not initialized
 *         -- Copy the data from EEPROM to shadow_addr
 *
 *  @param  shadow_addr : Destination buffer
 *          length      : Destination buffer length
 *
 *  @return  None
 *
 */
void EEPROM_Shadow_Init()
{
	uint32_t index;
    uint32_t val;
    uint8_t  retrycount = 10;
    uint8_t  *USER_Shadow_addr;

    /* Read the EEPROM First page to check the Magic number */
    /* Check Magic number is present in the EEPROM          */
	/* If it is present, the EEPROM is already initialized  */
	/* with the USER data                                   */

    while(retrycount > 0)
    {
    	TakeSPIBusLock();
    	EEPROM_SPI_ReadBuffer((uint8_t *)&val, 0x00, MAGIC_NUMBER_SIZE);
    	GiveSPIBusLock();
    	if (EEPROM_MAGIC_NUMBER == val)
    		break;

    	retrycount--;
    }

	if (EEPROM_MAGIC_NUMBER != val)
    {
		/* This is the first time writing, So initialize the shadow memory */
		Initialize_EEPlaceHolder();
		Init_Shadow_memory();
	}
	else
	{
     	/* USER Shadow start address */
	    USER_Shadow_addr = &Shadow_addr[0] + MAGIC_NUMBER_SIZE;

		/* Read the data from EEPROM and copy to Shadow Register */
        for (index = 0; index < (EE_USER_END - (EE_USER_START+1)); index ++)
		{
            /* Set the Placeholder EEPROM address with shadow_addr */
            EE_PLACEHOLDER_data[index].EE_info = &EE_user_elements[index];
		    EE_PLACEHOLDER_data[index].Shadow_EE_data_addr = USER_Shadow_addr ;

            /* Find out length of the element size */
		    val = ((EE_user_elements[index].Size + 1) *
			                (EE_user_elements[index].Redundancy + 1));

	        /* Read the EEPROM data */
		    TakeSPIBusLock();
		    EEPROM_SPI_ReadBuffer(USER_Shadow_addr, (USER_Shadow_addr - (&Shadow_addr[0])), val);
		    GiveSPIBusLock();

		    /* Increment shadow address */
			USER_Shadow_addr = USER_Shadow_addr +
                ((EE_user_elements[index].Size + 1) *
				(EE_user_elements[index].Redundancy + 1));

            EE_PLACEHOLDER_data[index].Update_data = FALSE;

            EE_PLACEHOLDER_data[index].EE_status = VALID_EE_DATA;
		}
	}
}

/** @brief Read EEPROM placeholder data
 *
 *  This function reads the EEPORM placeholder data from
 *  Shawdow memory.
 *
 *  @param  EE_PH_Index[IN] : EEPROM placeholder Index
 *          data[OUT]       : data buffer
 *          len[IN]         : data buffer size
 *
 *  @return INVALID_EE_DATA : Failure
 *          VALID_EE_DATA   : Success
 *
 */
uint8_t read_EE_Placeholder_data(uint16_t EE_PH_Index, uint8_t *data)
{
	uint32_t  addr;
	uint32_t index;
	uint8_t crc = 0;

	EE_PLACEHOLDER *EE_PH_data = &EE_PLACEHOLDER_data[EE_PH_Index];

	addr = (uint32_t)EE_PH_data->Shadow_EE_data_addr;

	/* Check the Shadow memory EEPlaceholder data is valid */
	if (INVALID_EE_DATA == EE_PH_data->EE_status)
		return INVALID_EE_DATA;

	/* Read the data from the EEPROM Shadow memory */
	for (index = 0; index < (EE_PH_data->EE_info->Redundancy + 1) ; index++)
    {
		/* Calculate the CRC */
		crc = HAL_CRC_Calculate(pv.hcrc, (uint32_t *)addr, EE_PH_data->EE_info->Size);

		/* validate the CRC value */
		if ( *(uint8_t*)(addr + EE_PH_data->EE_info->Size) == crc)
		{
            /* Copy the data to the user buffer */
			memcpy(data, (uint8_t *)addr, EE_PH_data->EE_info->Size);
			return VALID_EE_DATA;
		}

		/* Get the next Redundant address */
		addr = (uint32_t)(addr + (EE_PH_data->EE_info->Size + 1));
	}

	/* All the data are not valid and CRCs are failed */
	/* Check the default data is present in the Placeholder structure */
	if(NULL != EE_PH_data->EE_info->Default_data)
	{
		/* Get the Shadow address */
		addr = (uint32_t)EE_PH_data->Shadow_EE_data_addr;
    	memcpy(data, (uint8_t *)EE_PH_data->EE_info->Default_data, EE_PH_data->EE_info->Size);

    	EE_PH_data->EE_status = VALID_EE_DEFAULT;
	}
	else
		EE_PH_data->EE_status = INVALID_EE_DATA;

	return EE_PH_data->EE_status;
}


/** @brief Read EEPROM placeholder data
 *
 *  This function reads the EEPORM placeholder data from
 *  Shawdow memory.
 *
 *  @param  EE_PH_Index[IN] : EEPROM placeholder Index
 *          data[OUT]       : data buffer
 *          len[IN]         : data buffer size
 *
 *  @return FAILURE         : Invalid Parameter
 *          INVALID_EE_DATA : Failure
 *          VALID_EE_DATA   : Success
 *
 */
uint8_t Writeto_EE_Placeholder_data(uint16_t EE_PH_Index, uint8_t *data)
{
	uint32_t addr;
	uint8_t  *waddr;
	uint8_t  *taddr;
	uint16_t wlen  = 0;
	EepromOperations res;
	uint32_t index;
	uint16_t index1;
	uint8_t  crc = 0;

	EE_PLACEHOLDER *EE_PH_data = &EE_PLACEHOLDER_data[EE_PH_Index];

	addr = (uint32_t)EE_PH_data->Shadow_EE_data_addr;
	waddr = (uint8_t *) addr;

	/* Read the data from the EEPROM Shadow memory */
	for (index = 0; index < (EE_PH_data->EE_info->Redundancy + 1) ; index++)
    {
		/* Clear the shadow memory */
		memset((void *)addr, 0x00, EE_PH_data->EE_info->Size);

		/* Write data into the Shadow memory */
		memcpy((void *)addr, data, EE_PH_data->EE_info->Size);

		if ( 0 == crc)
		    /* Calculate the crc for the data */
		    crc = HAL_CRC_Calculate(pv.hcrc, (uint32_t *)data, EE_PH_data->EE_info->Size);

		addr = addr + EE_PH_data->EE_info->Size;
		wlen += EE_PH_data->EE_info->Size;

		/* Clear the shadow memory */
		memset((void *)addr, 0x00, sizeof(crc));

		/* Update the CRC value */
		*(uint8_t *)addr = crc;

		addr = addr + sizeof(crc);
		wlen += sizeof(crc);
	}

	index = 0;
	/* Update the initial status */
	EE_PH_data->EE_status = INVALID_EE_DATA;
	do
	{
		/* Write the Shadow data into the EEPROM */
		res = EEPROM_SPI_WriteBuffer(waddr,
				((uint32_t)waddr - (uint32_t)Shadow_addr), (uint16_t)wlen);

		/* If Write is success, read and verify */
		if (EEPROM_STATUS_COMPLETE == res)
		{
			taddr = (uint8_t *)(EEPROM_SDRAM_ADDRESS);

			/*Read EEPROM Data and confirm the data is present in the EEPROM */
			res = EEPROM_SPI_ReadBuffer(taddr,
					((uint32_t)waddr - (uint32_t)Shadow_addr),
					(uint16_t)wlen);

			for (index1=0; index1 < wlen; index1++)
			{
				if(waddr[index1] != taddr[index1])
					break;
			}
		}
		else
			res = -1;

		/* Both the data are equal, break */
		if (index1 == wlen)
		{
			EE_PH_data->EE_status = VALID_EE_DATA;
			break;
		}

		index++;
	}while(index < EEPROM_FAILURE_RETRY);

	return EE_PH_data->EE_status;
}

/** @brief Initialize the shadow memory
 *
 *  This function initialize the Shadow memory using the
 *  USER provided data
 *
 *  @param  None
 *
 *  @return None
 *
 */
void Init_Shadow_memory()
{
	uint16_t index;
	uint16_t index1;
	uint32_t magic_num = EEPROM_MAGIC_NUMBER;
	uint8_t  crc;

	/* First location is for the Magic number */
	uint32_t USER_Shadow_addr = (uint32_t) (&Shadow_addr[0] + MAGIC_NUMBER_SIZE);

	/* Clear the Shadow memory */
	memset((uint8_t *)USER_Shadow_addr, 0x00, (EEPROM_SIZE - MAGIC_NUMBER_SIZE));

	/* Copy the USER data to the Shadow address */
	for (index = 0; index < (EE_USER_END - (EE_USER_START+1)); index ++)
	{
     	/* Copy the default data to the Shadow memory */
	    for (index1 = 0; index1 < (EE_PLACEHOLDER_data[index].EE_info->Redundancy + 1) ; index1++)
        {

			if (TRUE == EE_PLACEHOLDER_data[index].EE_info->Default_data_enable)
            {
			    /* Copy the default data */
			    memcpy((uint8_t *)USER_Shadow_addr, EE_PLACEHOLDER_data[index].EE_info->Default_data,
                        EE_PLACEHOLDER_data[index].EE_info->Size);
			}

			/* Calculate the CRC */
            crc = HAL_CRC_Calculate(pv.hcrc ,(uint32_t *)USER_Shadow_addr, EE_PLACEHOLDER_data[index].EE_info->Size);

            /* Increment address with Data size */
            USER_Shadow_addr = USER_Shadow_addr + EE_PLACEHOLDER_data[index].EE_info->Size;

			/* Copy the CRC to the Shadow memory */
			memcpy((uint8_t *)USER_Shadow_addr, &crc, sizeof(crc));

			/* Increment address with CRC size */
			USER_Shadow_addr = USER_Shadow_addr + sizeof(crc);
		}

		/* Update the status */
		EE_PLACEHOLDER_data[index].EE_status = TRUE;
		EE_PLACEHOLDER_data[index].Update_data = TRUE;
	}

	/* Copy the Magic number to Shadow memory */
	memcpy(&Shadow_addr[0], &magic_num, MAGIC_NUMBER_SIZE);

	/* Write the Shadow data into the EEPROM */
	TakeSPIBusLock();

	/* Erase the EEPROM data */
	EEPROM_eraseAll();

	EEPROM_SPI_WriteBuffer(&Shadow_addr[0], 0x00, EEPROM_PAGESIZE);
	GiveSPIBusLock();

}

/** @brief EEPROM DB Initialization
 *
 *  This function takes the default configuration from the config header
 *  file and store the data into the DB.
 *
 *  @param  None
 *
 *  @return None
 *
 */
void EEPROM_DB_Init(void)
{

	/* Initialize the shadow memory */
	EEPROM_Shadow_Init();
}

/** @brief EEPROM Platform service task
 *
 *  This function starts the EEPROM Platform Service
 *  task.
 *
 *  @param  pvParameters - Private data pointer
 *
 *  @return none
 *
 */
void EE_ServiceStart(void *pvParameters)
{
	uint32_t cNotifiedValue;
	uint32_t addr;
	uint8_t  *eebuf;
	uint8_t  res = 0;
	uint8_t  func;
	//uint8_t  len;

	while(1)
	{

#if (SDK_SERVICE_WATCHDOG == PS_ENABLE)
		/* Ping the watchdog timer */
		res = SetWD(WD_PING_EE_TASK);

		if(res != SUCCESS)
		{
			LOGERROR("EEPROM Task failed to ping the WD Timer\r\n");
		}
#endif
		/* Wait for receiving the Event from the SDK API */
		res = xTaskNotifyWait( CLEAR_BITS_ONENTRY, ULONG_MAX , &cNotifiedValue, PS_EE_TASK_TIMEOUT);

		if(res != pdTRUE)
		{
			continue;
		}

		/* Process the SDK API message */
		if (cNotifiedValue & (1<<EE_MSG_EVENT))
		{
				/* Receive Message from the EEPROM SDK API */
				res = xQueueReceive(pv.eetxq, &eebuf , ( TickType_t ) Q_TICKSTOWAIT );

				if(res != pdTRUE)
				{
					break;
				}

				func = eebuf[EE_FUNC_INDEX];

				/* Update the result */
				eebuf[EE_RESULT_INDEX] = SUCCESS;

				switch(func)
				{
					case EE_GET:

						/* Get the data address from the packet */
					    memcpy(&addr, (uint8_t *)&eebuf[EE_PH_DATA_INDEX], sizeof(addr));

					    TakeSPIBusLock();
					    /* Read the data from the EEPROM */
					    res = read_EE_Placeholder_data(eebuf[EE_PH_ID_INDEX],
					    		                    (uint8_t *)addr);
					    GiveSPIBusLock();

					    /* Update the Read status to DB */
					    Set_DL(EEPROM_READ_STATUS, &res);

						break;

					case EE_SET:

                        /* Get the data address from the packet */
					    memcpy(&addr, (uint8_t *)&eebuf[EE_PH_DATA_INDEX], sizeof(addr));

					    TakeSPIBusLock();
					    /* Write the data into the EEPROM */
					    res = Writeto_EE_Placeholder_data(eebuf[EE_PH_ID_INDEX],
					    		                    (uint8_t *)addr);
					    GiveSPIBusLock();

					    /* Update the Write status to DB */
					    Set_DL(EEPROM_WRITE_STATUS, &res);

						break;

					default:
						eebuf[EE_RESULT_INDEX] = FAILURE;
						break;
				}

				/* Send the response to the SDK API */
				xQueueSend( pv.eerxq, ( void * )&eebuf, ( TickType_t ) Q_TICKSTOWAIT );
		}
	}
}

/** @brief Start EEPROM Platform service
 *
 *  This function initializes the following modules
 *     ===> EEPROM DB
 *     ===> EEPROM platform driver
 *     ===> EEPROM software resources
 *     ===> Start the EEPROM service
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *         -1 : PS_FAILURE
 *
 */
int8_t EE_PS_Start(void)
{
	BaseType_t xReturned;

	/* Create EEPROM TxQ for receiving value from the SDK API */
	pv.eetxq = xQueueCreate( EE_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.eetxq)
	{
		LOGERROR("Unable to initialize the EEPROM TxQ \r\n");
		return PS_FAILURE;
	}

	/* Create EEPROM RxQ for sending return value to the SDK API */
	pv.eerxq = xQueueCreate( EE_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.eerxq)
	{
		LOGERROR("Unable to initialize the EEPROM RxQ \r\n");
		return PS_FAILURE;
	}

	/* Create the EEPROM task */
	xReturned = xTaskCreate(EE_ServiceStart, "EE_Service",
			EE_TASK_SIZE, NULL, PS_EE_TASK_PRIORITY,   &pv.ee_task);

    if( pdPASS != xReturned )
    {
		LOGERROR("Unable to initialize the EEPROM Task\r\n");
		return PS_FAILURE;
    }

    LOGERROR("EEPROM Platform Service task created Successfully \r\n");

	return PS_SUCCESS;
}
#endif	//SDK_SERVICE_EEPROM

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
