/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : SDK_API.c
   Author             : Rajesh
   Version            : V0.0.0.1
   Origination Date   : 21/01/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the SDK API for all the peripheral devices

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.0             21/01/2022           Rajesh

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../Inc/Common.h"
/* USER CODE END Includes */

/* USER CODE BEGIN PV */
extern pvt_variable pv;
/* USER CODE END PV */

#if (SDK_SERVICE_WARNING_LIGHT == PS_ENABLE)
/** @brief Sends Warning light data
 *
 *  This function sends the control data to the Warning light device
 *  through the Platform service.
 *
 *
 *  @param  wl_number[IN]        : Warning light number
 *          blinking_period[IN]  : Warning light blink period
 *          duty_cycle[IN]       : Warning light duty cycle
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 *           2 : MEM_FAILURE
 */
uint8_t ActWLT(uint8_t wl_number, uint16_t blinking_period, uint8_t duty_cycle)
{
	uint8_t *buff;
    uint8_t res;

    /* Validate the wl_number parameter */
    if (wl_number > MAX_PLATFORM_SUPPORTED_WL)
    	return FAILURE;

    /* Validate the blinking period */
    if ((WL_MIN_BLK_TIME > blinking_period) ||
    		(WL_MAX_BLK_TIME < blinking_period))
	{
    	return FAILURE;
	}

    /*  Validate the duty cycle */
    if ((WL_MIN_PWM_DC > duty_cycle) ||
    		(WL_MAX_PWM_DC < duty_cycle))
	{
    	return FAILURE;
	}

   	/* Get the buffer from the Memory */
   	buff = (uint8_t *) ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[WL_FUNC_INDEX]  = WL_ACT;

	/* WL Number */
    buff[WL_NUM_INDEX+1] = wl_number;

    /* Duty cycle */
    buff[WL_DC_INDEX]    = duty_cycle;

    /* Blink Period */
    buff[WL_BL_INDEX]    = (uint8_t )((blinking_period >> 8) & 0xFF);
    buff[WL_BL_INDEX+1]  = (uint8_t ) (blinking_period & 0xFF);

	/* Send the data into the Warning Light Q */
	if (xQueueSend( pv.wltxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the WL task to update the WL data */
	if (xTaskNotify( pv.wl_task, (1<<WL_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data */
    /* Send the data to the Warning light platform service layer */
	if (xQueueReceive(pv.wlrxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	res = buff[WL_RESULT_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return res;
}

/** @brief Sends Warning light data
 *
 *  This function sends the WL ON/OFF data to the Warning light device
 *  through the Platform service.
 *
 *
 *  @param  wl_number[IN]        : Warning Light Number
 *          state[IN]            : ON/OFF State
 *
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 *           2 : MEM_FAILURE
 */
uint8_t SetWLT(uint8_t wl_number, uint8_t *cbuf)
{
	uint8_t *buff;
	uint8_t res;

	/* Validate the wl_number parameter */
	if (wl_number > MAX_PLATFORM_SUPPORTED_WL)
		return FAILURE;

	/* Validate the state variable */
	if ((WL_ON != cbuf[WL_STATE_INDEX]) && (WL_OFF != cbuf[WL_STATE_INDEX]))
		return FAILURE;

   	/* Get the buffer from the Memory */
   	buff = (uint8_t *) ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[WL_FUNC_INDEX] = WL_SET;

	/* WL Number */
    buff[WL_NUM_INDEX+1] = cbuf[WL_NUM_INDEX];

    /* State value */
    buff[WL_STATE_INDEX+1] = cbuf[WL_STATE_INDEX];

    /* Duty Cycle  */
    buff[WL_DC_INDEX+1] = cbuf[WL_DC_INDEX];

    /* Blinking period */
    buff[WL_BL_INDEX+1] = cbuf[WL_BL_INDEX];
    buff[WL_BL_INDEX+2] = cbuf[WL_BL_INDEX+1];

	/* Send the data into the Warning Light Q */
	if (xQueueSend( pv.wltxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the WL task to update the WL data */
	if (xTaskNotify( pv.wl_task, (1<<WL_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data */
    /* Send the data to the Warning light platform service layer */
	if (xQueueReceive(pv.wlrxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	res = buff[WL_RESULT_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return res;
}

#endif	//SDK_SERVICE_WARNING_LIGHT

#if (SDK_SERVICE_KEYPAD == PS_ENABLE)
/** @brief Sends keypad data
 *
 *  This function sends the keypad control data to the
 *  Platform service.
 *
 *
 *  @param  Keypad_ID[IN]        : KEYPAD_BACKLIGHT
 *          State[IN]            : KEY_BACKLIGHT_ON / KEY_BACKLIGHT_OFF
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 */
uint8_t SetKPD(uint8_t keypad_id, uint8_t state)
{
	uint8_t *buff;
    uint8_t res;

	/* Validate the keypad state */
	if ((KEY_BACKLIGHT_ON != state) && (KEY_BACKLIGHT_OFF != state))
		return FAILURE;

	/* Get the buffer from the Memory */
	buff = (uint8_t *) ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[0] = KEYPAD_SET;
    buff[1] = state;

	/* Send the data into the Keypad Q */
	if (xQueueSend( pv.kpdtxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the keypad task to update the keypad data */
	if (xTaskNotify( pv.kpd_task, (1<<KEYPAD_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data */
    /* Send the data to the keypad platform service layer */
	if (xQueueReceive(pv.kpdrxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	res = buff[KPD_RESULT_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return res;
}

#endif	//SDK_SERVICE_KEYPAD

#if (SDK_SERVICE_RTC == PS_ENABLE)
/** @brief Set RTC / ALARM time
 *
 *  This function sends the control data to the RTC Platform service.
 *
 *  @param  func[IN]             : Input Functions
 *                                 TIME
 *                                 ALARM_A
 *                                 ALARM_B
 *
 *          param[IN]            : Time / Alarm parameter
 *
 *          format[IN]           : FORMAT_12_HOUR
 *                                 FORMAT_24_HOUR
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 */
uint8_t SetRTC(RTC_Func func, rtc_ps *param, uint8_t format)
{
	uint8_t *buff;
    uint8_t ret;

    if (NULL == param)
    	return FAILURE;

    /* Validate the func parameter */
    if ((TIME != func) && (ALARM_A != func) && (ALARM_B != func))
        return FAILURE;

    /* Validate the seconds */
    if (!((RTC_MIN_VAL <= param->Seconds) && (RTC_MAX_VAL > param->Seconds)))
        return FAILURE;

    /* Validate the Minutes */
    if (!((RTC_MIN_VAL <= param->Minutes) && (RTC_MAX_VAL > param->Minutes)))
        return FAILURE;

    /* Validate the Hours */
    if (!(((FORMAT_12_HOUR == format) && (RTC_MIN_VAL <= param->Hours) && (RTC_12H_MAX_VAL >= param->Hours)) ||
    		((FORMAT_24_HOUR == format) && (RTC_MIN_VAL <= param->Hours) && (RTC_24H_MAX_VAL > param->Hours))))
        return FAILURE;

    /* Validate the YEAR */
    if (!((RTC_YEAR_MIN_VAL <= param->Year) && (RTC_YEAR_MAX_VAL >= param->Year)))
        return FAILURE;

    /* Validate the WeekDay */
    if (!((RTC_WEEKDAY_MIN_VAL <= param->WeekDay) && (RTC_WEEKDAY_MAX_VAL >= param->WeekDay)))
        return FAILURE;

    /* Get the buffer */
	buff =  (uint8_t *)ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	if(func == TIME)
	{
		buff[RTC_FUNC_INDEX] = RTC_SET;
	}
	else if(func == ALARM_A)
	{
		buff[RTC_FUNC_INDEX] = RTC_SET_ALARM_A;
	}
	else if(func == ALARM_B)
	{
		buff[RTC_FUNC_INDEX] = RTC_SET_ALARM_B;
	}

	buff[RTC_SEC_INDEX] 		= param->Seconds;
	buff[RTC_MIN_INDEX] 		= param->Minutes;
	buff[RTC_HOUR_INDEX] 		= param->Hours;
	buff[RTC_DATE_INDEX] 		= param->Date;
	buff[RTC_MONTH_INDEX] 		= param->Month;
	buff[RTC_YEAR_INDEX] 		= param->Year;
	buff[RTC_WEEKDAY_INDEX] 	= param->WeekDay;
	buff[RTC_TIME_FORMAT_INDEX] = param->time_format;

	/* Send the data into the RTC Platform Service Q */
	if (xQueueSend( pv.rtctxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the RTC task to update the time */
	if (xTaskNotify( pv.rtc_task, (1<<RTC_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data                 */
    /* Send the data to the RTC platform service layer */
	if (xQueueReceive(pv.rtcrxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	ret = buff[RTC_RESULT_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return ret;
}


/** @brief Get RTC / ALARM time
 *
 *  This function gets RTC/ALARM data RTC through the Platform service.
 *
 *  @param  func[IN]             : Input Functions
 *                                 TIME
 *                                 ALARM_A
 *                                 ALARM_B
 *
 *          param[OUT]           : Time / Alarm parameter
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 */
uint8_t GetRTC(RTC_Func func, rtc_ps *param)
{
	uint8_t *buff;
    uint8_t ret;

    if (NULL == param)
    	return FAILURE;

    /* Get the buffer */
	buff =  (uint8_t *)ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[RTC_FUNC_INDEX] = func;

	/* Send the data into the RTC Platform Service Q */
	if (xQueueSend( pv.rtctxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the RTC task to update the time */
	if (xTaskNotify( pv.rtc_task, (1<<RTC_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data              */
    /* Send the data to the RTC platform service layer */
	if (xQueueReceive(pv.rtcrxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	ret = buff[RTC_RESULT_INDEX];

	if (SUCCESS != ret)
	{
		/* Update the data info the USER buffer */
		param->Seconds = buff[RTC_SEC_INDEX];
		param->Minutes = buff[RTC_MIN_INDEX];
		param->Hours   = buff[RTC_HOUR_INDEX];
		param->Date    = buff[RTC_DATE_INDEX];
		param->Month   = buff[RTC_MONTH_INDEX];
		param->Year    = buff[RTC_YEAR_INDEX];
		param->WeekDay = buff[RTC_WEEKDAY_INDEX];
	}

	/* Release the buffer */
	ai430_memory_free(buff);

	return ret;
}

#endif	//SDK_SERVICE_RTC

#if (SDK_SERVICE_DIGITAL_OUTPUT == PS_ENABLE)

/** @brief Digital Output SET API
 *
 *  This function sends the Digital Output setting to hardware
 *  through the Platform service.
 *
 *
 *  @param  state[IN]           : Source buffer
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 */
uint8_t SetDIO(DIO_Func func, uint8_t state)
{
	uint8_t *buff;
    uint8_t ret;

    /* Validate the func parameter */
    if ((SET_DIO_STATE != func) && (SET_DIO_CFG != func))
        return FAILURE;

    /* Validate the Mode Parameter */
    if ((SET_DIO_STATE == func) &&
    		 (CONF_DIGITAL_OUTPUT_ON != state) &&
    		 (CONF_DIGITAL_OUTPUT_OFF != state))
    {
          return FAILURE;
    }
    else
    /* Validate the Conversion time Parameter */
    if ((SET_DIO_CFG == func) &&
    		 (CONF_LOW_SIDE_DRIVER != state) &&
    		 (CONF_HIGH_SIDE_DRIVER != state) &&
			 (CONF_OPEN_DRIVE_DRIVER != state))
    {
          return FAILURE;
    }

    /* Get the buffer */
	buff =  (uint8_t *)ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[DIO_FUNC_INDEX]  = DO_SET;
	buff[DIO_SUB_FUNC_INDEX]  = func;
	buff[DIO_STATE_INDEX] = state;

	/* Send the data into the DIO Platform Service Q */
	if (xQueueSend( pv.dotxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the DIO task to update the DIO data */
	if (xTaskNotify( pv.do_task, (1<<DIO_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data                 */
    /* Send the data to the DIO platform service layer */
	if (xQueueReceive(pv.dorxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	ret = buff[DIO_RESULT_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return ret;
}

/** @brief Digital Output Read API
 *
 *  This function reads the state of the Digital Output
 *  from the hardware through the Platform service.
 *
 *
 *  @param  state[Out]        : State of the Digital Output
 *
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 */
uint8_t GetDIO(uint8_t *state)
{
	uint8_t *buff;
    uint8_t ret;

    /* Initialize the state */
    *state = 0;

    /* Get the buffer */
	buff =  (uint8_t *)ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[DIO_FUNC_INDEX] = DO_GET;

	/* Send the data into the DIO Platform Service Q */
	if (xQueueSend( pv.dotxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the DIO task to update the DIO data */
	if (xTaskNotify( pv.do_task, (1<<DIO_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data                 */
    /* Send the data to the DIO platform service layer */
	if (xQueueReceive(pv.dorxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	ret = buff[DIO_RESULT_INDEX];

	/* return the state of the Digital Output */
	*state = buff[DIO_STATE_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return ret;
}

#endif	//SDK_SERVICE_DIGITAL_OUTPUT

#if (SDK_SERVICE_LIGHT_SENSOR == PS_ENABLE)
/** @brief Set Light sensor configuration
 *
 *  This function sends the control data to the Light sensor light device
 *  through the Platform service.
 *
 *
 *  @param  func[IN]             : Input Functions
 *                                 LS_SET_MODE,
 *                                 LS_SET_CONVERSION_TIME
 *
 *          value[IN]            : Value for the function
 *                                 LS_SET_MODE
 *                                 ===========
 *                                 LS_SINGLE_MODE_CONV
 *                                 LS_CONTINOUS_MODE_CONV
 *                                 LS_SHUTDOWN
 *
 *                                 LS_SET_CONVERSION_TIME
 *                                 =====================
 *                                 LS_CONVERSION_TIME_100
 *                                 LS_CONVERSION_TIME_800
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 */

uint8_t SetLTS(uint8_t func, uint8_t value)
{
	uint8_t *buff;
    uint8_t ret;

    /* Validate the func parameter */
    if ((LS_SET_MODE != func) && (LS_SET_CONVERSION_TIME != func))
        return FAILURE;

    /* Validate the Mode Parameter */
    if ((LS_SET_MODE == func) &&
    		 (LS_SINGLE_MODE_CONV != value) &&
    		 (LS_CONTINUOUS_MODE_CONV != value) &&
			 (LS_SHUTDOWN != value))
    {
          return FAILURE;
    }
    else
    /* Validate the Conversion time Parameter */
    if ((LS_SET_CONVERSION_TIME == func) &&
    		 (LS_CONVERSION_TIME_100 != value) &&
    		 (LS_CONVERSION_TIME_800 != value))
    {
          return FAILURE;
    }

    /* Get the buffer */
	buff =  (uint8_t *)ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[LS_FUNC_INDEX] 		= LS_SET;
	buff[LS_SUB_FUNC_INDEX] 	= func;
	buff[LS_SUB_FUNC_VAL_INDEX] = value;

	/* Send the data into the LTS Platform Service Q */
	if (xQueueSend( pv.lstxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the LTS task to update the LTS data */
	if (xTaskNotify( pv.ls_task, (1<<LS_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data                 */
    /* Send the data to the LTS platform service layer */
	if (xQueueReceive(pv.lsrxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	ret = buff[LS_RESULT_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return ret;
}


/** @brief Set Light sensor configuration
 *
 *  This function sends the control data to the Light sensor light device
 *  through the Platform service.
 *
 *
 *  @param  func[IN]             : Input Functions
 *                                 LS_SET_MODE,
 *                                 LS_SET_CONVERSION_TIME
 *
 *          value[IN]            : Value for the function
 *                                 LS_SET_MODE
 *                                 ===========
 *                                 LS_SINGLE_MODE_CONV
 *                                 LS_CONTINOUS_MODE_CONV
 *                                 LS_SHUTDOWN
 *
 *                                 LS_SET_CONVERSION_TIME
 *                                 =====================
 *                                 LS_CONVERSION_TIME_100
 *                                 LS_CONVERSION_TIME_800
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 */
uint8_t GetLTS(uint8_t func, uint8_t *value)
{
	uint8_t *buff;
    uint8_t ret;

    /* Validate the pointer parameter */
    if (NULL == value)
    	return FAILURE;

    /* Validate the func parameter */
    if ((LS_SET_MODE != func) &&
    		(LS_SET_CONVERSION_TIME != func) &&
			(LS_READ_DATA != func))
        return FAILURE;

    /* Get the buffer */
	buff =  (uint8_t *)ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[LS_FUNC_INDEX] 	= LS_GET;
	buff[LS_SUB_FUNC_INDEX] = func;

	/* Send the data into the LTS Platform Service Q */
	if (xQueueSend( pv.lstxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the LTS task to update the LTS data */
	if (xTaskNotify( pv.ls_task, (1<<LS_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data                 */
    /* Send the data to the LTS platform service layer */
	if (xQueueReceive(pv.lsrxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	ret = buff[LS_RESULT_INDEX];

	/* Copy the data to the user buffer */
    if ((LS_SET_MODE == func) && (LS_SET_CONVERSION_TIME == func))
    {
        memcpy(value, &buff[LS_SUB_FUNC_INDEX], sizeof(uint16_t));
    }
    else
    {
        /* Read Light sensor data */
    	memcpy(value, &buff[LS_SUB_FUNC_INDEX], sizeof(float));
    }

	/* Release the buffer */
	ai430_memory_free(buff);

	return ret;
}

#endif	//SDK_SERVICE_LIGHT_SENSOR


#if (SDK_SERVICE_WATCHDOG == PS_ENABLE)

/** @brief Watchdog SET API
 *
 *  This function sends the Watchdog setting to hardware
 *  through the Platform service.
 *
 *
 *  @param  ping_id[IN]           : Watchdog ping id
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 */
uint8_t SetWD(uint8_t ping_id)
{
	uint8_t *buff;
    uint8_t ret;

    /* Get the buffer */
	buff =  (uint8_t *)ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[WD_FUNC_INDEX] 	= WD_SET;
	buff[WD_FEED_ID_INDEX] 	= ping_id;

	/* Send the data into the Watchdog Platform Service Q */
	if (xQueueSend( pv.wdtxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the Watchdog task to update the watchdog data */
	if (xTaskNotify( pv.wd_task, (1<<WD_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data                 */
    /* Send the data to the Watchdog platform service layer */
	if (xQueueReceive(pv.wdrxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	ret = buff[WD_RESULT_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return ret;
}
#endif	//SDK_SERVICE_WATCHDOG

#if (SDK_SERVICE_LED == PS_ENABLE)
/** @brief Sends LED data
 *
 *  This function sends the control data to the LED device
 *  through the Platform service.
 *
 *
 *  @param  led_number[IN]        : led number
 *          blinking_period[IN]   : led blink period
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 *           2 : MEM_FAILURE
 */
uint8_t ActLED(uint8_t led_number, uint16_t blinking_period)
{
	uint8_t *buff;
    uint8_t res;

	/* Validate the LED number */
	if (led_number > MAX_PLATFORM_SUPPORTED_LED)
		return FAILURE;

    /* Validate the blinking period */
	if(!((LED_MIN_BLK_TIME <= blinking_period) && (LED_MAX_BLK_TIME >  blinking_period)))
		return FAILURE;

   	/* Get the buffer from the Memory */
   	buff = (uint8_t *) ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[LED_FUNC_INDEX]  = LED_ACT;

	/* LED Number */
    buff[LED_NUM_INDEX+1] = led_number;

    /* Blink Period */
    buff[LED_BLK_INDEX]    = (uint8_t )((blinking_period >> 8) & 0xFF);
    buff[LED_BLK_INDEX+1]  = (uint8_t ) (blinking_period & 0xFF);

	/* Send the data into the LED TxQ */
	if (xQueueSend(pv.ledtxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the LED task to update the LED data */
	if (xTaskNotify(pv.led_task, (1<<LED_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data */
    /* Send the data to the LED platform service layer */
	if (xQueueReceive(pv.ledrxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	res = buff[LED_RESULT_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return res;
}

/** @brief Sends LED data
 *
 *  This function sends the LED ON/OFF data to the LED device
 *  through the Platform service.
 *
 *
 *  @param  led_number[IN]        : led Number
 *          cbuf[IN]              : led parameter
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 *           2 : MEM_FAILURE
 */
uint8_t SetLED(uint8_t led_number, uint8_t *lbuf)
{
	uint8_t *buff;
	uint8_t res;

	/* Validate the wl_number parameter */
	if (led_number > MAX_PLATFORM_SUPPORTED_LED)
		return FAILURE;

	/* Validate the state variable */
	if ((LED_ON != lbuf[LED_STATE_INDEX]) && (LED_OFF != lbuf[LED_STATE_INDEX]))
		return FAILURE;

   	/* Get the buffer from the Memory */
   	buff = (uint8_t *) ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[LED_FUNC_INDEX] = LED_SET;

	/* LED Number */
    buff[LED_NUM_INDEX+1] = lbuf[LED_NUM_INDEX];

    /* State value */
    buff[LED_STATE_INDEX+1] = lbuf[LED_STATE_INDEX];

    /* Blinking period */
    buff[LED_BLK_INDEX+1] = lbuf[LED_BLK_INDEX];
    buff[LED_BLK_INDEX+2] = lbuf[LED_BLK_INDEX+1];

	/* Send the data into the LED TxQ */
	if (xQueueSend( pv.ledtxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the LED task to update the LED data */
	if (xTaskNotify( pv.led_task, (1<<LED_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data */
    /* Send the data to the LED platform service layer */
	if (xQueueReceive(pv.ledrxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	res = buff[LED_RESULT_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return res;
}

#endif	//SDK_SERVICE_LED


#if (SDK_SERVICE_CFG_INPUT == PS_ENABLE)

/** @brief Configurable SET API
 *
 *  This function sends the configurable input setting to hardware
 *  through the Platform service.
 *
 *  @param  set_state[IN]        : CI_SET_INPUT_TYPE/
 *  							 : CI_SET_NUM_SAMPLE
 *
 *  		ci_type_idx[IN]	     : CONF_INPUT_TYPE_01/
 *  							 : CONF_INPUT_TYPE_02/
 *  							 : CONF_INPUT_TYPE_03
 *  							 :
 *  							 : CONF_INPUT_TYPE_06
 *  		value				 : configurable input value
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 */
uint8_t SetCGI(uint8_t set_state, uint8_t ci_type_idx, uint8_t value)
{
	uint8_t *buff;
    uint8_t ret;

    /* Get the buffer */
	buff =  (uint8_t *)ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[CI_FUNC_INDEX]   = CI_SET;
	buff[CI_SET_ID_INDEX] = set_state;
	buff[CI_TYPE_INDEX]   = ci_type_idx;
	buff[CI_VALUE_INDEX]  = value;

	/* Send the data into the CI Platform Service Q */
	if (xQueueSend( pv.citxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the CI task to update the CI data */
	if (xTaskNotify( pv.ci_task, (1<<CI_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data                 */
    /* Send the data to the CI platform service layer */
	if (xQueueReceive(pv.cirxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	ret = buff[CI_RESULT_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return ret;
}

/** @brief configurable input Read API
 *
 *  This function reads the state of the configurable input
 *  from the hardware through the Platform service.
 *
 *
 *  @param  value[Out]        : configurable input value
 *
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 */
uint8_t GetCGI(uint8_t get_state, uint8_t ci_type_idx, uint8_t *value)
{
	uint8_t *buff;
    uint8_t ret;

    /* Initialize the value to zero */
    *value = 0;

    /* Get the buffer */
	buff =  (uint8_t *)ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[CI_FUNC_INDEX]   = CI_GET;
	buff[CI_GET_ID_INDEX] = get_state;
	buff[CI_TYPE_INDEX]   = ci_type_idx;

	/* Send the data into the CI Platform Service Q */
	if (xQueueSend( pv.citxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the CI task to update the CI data */
	if (xTaskNotify( pv.ci_task, (1<<CI_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data                 */
    /* Send the data to the CI platform service layer */
	if (xQueueReceive(pv.cirxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	ret = buff[CI_RESULT_INDEX];

	/* return the value of the configurable input */
	*value = buff[CI_GET_ID_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return ret;
}

#endif	//SDK_SERVICE_CFG_INPUT

#if (SDK_SERVICE_LCD == PS_ENABLE)
/** @brief Sends LCD data
 *
 *  This function sends the LCD control data to the
 *  Platform service.
 *
 *
 *  @param  lcd_id[IN]   : SET_LCD_STATE/SET_LCD_BRIGHTNESS
 *          value[IN]    : value
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 */
uint8_t SetLCD(LCD_Func lcd_fun, uint8_t value)
{
	uint8_t *buff;
    uint8_t res;

	/* Validate the keypad state */
	if ((SET_LCD_STATE != lcd_fun) && (SET_LCD_BRIGHTNESS != lcd_fun))
		return FAILURE;

    /* Validate the LCD State */
    if ((SET_LCD_STATE == lcd_fun) &&
    		 (LCD_CONF_ON != value) &&
    		 (LCD_CONF_OFF != value))
    {
          return FAILURE;
    }
    else
    	/* Validate the LCD Brightness */
    if ((SET_LCD_BRIGHTNESS == lcd_fun) &&
    		(MIN_LCD_BRIGHTNESS >= value) &&
			(MAX_LCD_BRIGHTNESS <= value))
    {
          return FAILURE;
    }

	/* Get the buffer from the Memory */
	buff = (uint8_t *) ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[LCD_FUNC_INDEX] 	 = SET_LCD;
	buff[LCD_SUB_FUNC_INDEX] = lcd_fun;
    buff[LCD_STATE_INDEX] 	 = value;

	/* Send the data into the LCD Q */
	if (xQueueSend( pv.lcdtxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the LCD task to update the LCD data */
	if (xTaskNotify( pv.lcd_task, (1<<LCD_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data */
    /* Send the data to the LCD platform service layer */
	if (xQueueReceive(pv.lcdrxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	res = buff[LCD_RESULT_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return res;
}

/** @brief Get LCD configuration
 *
 *  This function sends the control data to the LCD device
 *  through the Platform service.
 *
 *  @param  func[IN]             : SET_LCD_STATE
 *                                 SET_LCD_BRIGHTNESS,
 *
 *          value[IN]            : Value for the function
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 */
uint8_t GetLCD(LCD_Func lcd_fun, uint8_t *value)
{
	uint8_t *buff;
    uint8_t ret;

    /* Validate the pointer parameter */
    if (NULL == value)
    	return FAILURE;

    /* Validate the func parameter */
    if ((SET_LCD_STATE != lcd_fun) &&
    		(SET_LCD_BRIGHTNESS != lcd_fun))
        return FAILURE;

    /* Get the buffer */
	buff =  (uint8_t *)ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[LCD_FUNC_INDEX] 	 = GET_LCD;
	buff[LCD_SUB_FUNC_INDEX] = lcd_fun;

	/* Send the data into the LCD Platform Service Q */
	if (xQueueSend( pv.lcdtxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the LCD task to update the LCD data */
	if (xTaskNotify( pv.lcd_task, (1<<LCD_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data                 */
    /* Send the data to the LCD platform service layer */
	if (xQueueReceive(pv.lcdrxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	ret = buff[LCD_RESULT_INDEX];

	/* Copy the data to the user buffer */
    if ((SET_LCD_STATE == lcd_fun) && (SET_LCD_BRIGHTNESS == lcd_fun))
    {
        memcpy(value, &buff[LCD_STATE_INDEX], sizeof(uint8_t));
    }

	/* Release the buffer */
	ai430_memory_free(buff);

	return ret;
}

#endif	//SDK_SERVICE_LCD

#if (SDK_SERVICE_PM == PS_ENABLE)
/** @brief Set the Power Mode
 *
 *  This function sends power management mode to the
 *  Platform service.
 *
 *
 *  @param  mode[IN]  : SLEEP
 *                      STOP
 *                      STANDBY
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 */
uint8_t SetPWR(uint8_t mode)
{
	uint8_t *buff;
    uint8_t res;

	/* Validate the Power Management Mode */
	if ((PM_SLEEP_MODE != mode) && (PM_STOP_MODE != mode)
			&& (PM_STANDBY_MODE != mode))
		return FAILURE;

	/* Get the buffer from the Memory */
	buff = (uint8_t *) ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[PM_FUNC_INDEX]     = PM_SET;
    buff[PM_MODE_INDEX] 	= mode;

	/* Send the data into the Power Mode Q */
	if (xQueueSend( pv.pmtxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the Power mode task to update the Power mode data */
	if (xTaskNotify( pv.pm_task, (1<<PM_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data from the PM Service*/
	if (xQueueReceive(pv.pmrxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	res = buff[PM_RESULT_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return res;
}

#endif	//SDK_SERVICE_PM

#if (SDK_SERVICE_EEPROM == PS_ENABLE)
/** @brief Write EEPROM
 *
 *  This function sends the write data Command into the EEPROM
 *  Platform service.
 *
 *
 *  @param  eeph[IN]  : EEPROM Placeholder ID
 *          data[IN]  : Data to be written
 *          len [IN]  : Length of the data buffer
 *
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 */
uint8_t SetERM(uint16_t eeph, uint8_t *data)
{
	uint8_t *buff;
    uint8_t res;
    uint32_t addr;

	if (!((EE_MAX_PH >= eeph) && (EE_MIN_PH <= eeph)))
		return FAILURE;

	/* Get the buffer from the Memory */
	buff = (uint8_t *) ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[EE_FUNC_INDEX]     = EE_SET;
    buff[EE_PH_ID_INDEX] 	= eeph;

    /* Copy the data address to the message buffer */
    addr = (uint32_t) data;
    memcpy(&buff[EE_PH_DATA_INDEX], &addr, sizeof(addr));

    /* Copy the data Len to the message buffer */
    //memcpy(&buff[EE_PH_LEN_INDEX + sizeof(addr)], &len, sizeof(len));

	/* Send the data into the EEPROM Q */
	if (xQueueSend( pv.eetxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the EEPROM task */
	if (xTaskNotify( pv.ee_task, (1<<EE_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data from the EEPROM Service*/
	if (xQueueReceive(pv.eerxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	res = buff[EE_RESULT_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return res;
}

/** @brief READ EEPROM
 *
 *  This function sends the read data Command into the EEPROM
 *  Platform service.
 *
 *
 *  @param  eeph[IN]  : EEPROM Placeholder ID
 *          data[OUT] : Data to be written
 *          len [IN]  : Length of the data buffer
 *
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 */
uint8_t GetERM(uint16_t eeph, uint8_t *data)
{
	uint8_t  *buff;
    uint8_t  res;
    uint32_t addr;

	if (!((EE_MAX_PH >= eeph) && (EE_MIN_PH <= eeph)))
		return FAILURE;

	/* Get the buffer from the Memory */
	buff = (uint8_t *) ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;


	/* Construct the buffer */
	buff[EE_FUNC_INDEX]     = EE_GET;
    buff[EE_PH_ID_INDEX] 	= eeph;

    /* Copy the data address to the message buffer */
    addr = (uint32_t) data;
    memcpy(&buff[EE_PH_DATA_INDEX], &addr, sizeof(addr));

	/* Send the data into the EEPROM Q */
	if (xQueueSend( pv.eetxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the EEPROM task */
	if (xTaskNotify( pv.ee_task, (1<<EE_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data from the PM Service*/
	if (xQueueReceive(pv.eerxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	res = buff[EE_RESULT_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return res;
}
#endif	//SDK_SERVICE_EEPROM

#if (SDK_SERVICE_BLE == PS_ENABLE)
/** @brief Send BLE data
 *
 *  This function sends the BLE data into the BLE
 *  Platform service.
 *
 *  @param  data[IN]  : Data to be Send
 *          len [IN]  : Length of the data buffer
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 */
uint8_t SetBLE(int8_t *data, uint8_t len)
{
	uint8_t *buff;
    uint8_t res;
    uint32_t addr;

	/* Get the buffer from the Memory */
	buff = (uint8_t *) ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[BLE_FUNC_INDEX]     = BLE_SET;

    /* Copy the data address to the message buffer */
    addr = (uint32_t) data;
    memcpy(&buff[BLE_DATA_INDEX], &addr, sizeof(addr));

    /* Copy the data Len to the message buffer */
    memcpy(&buff[BLE_LEN_INDEX + sizeof(addr)], &len, sizeof(len));

	/* Send the data into the BLE Q */
	if (xQueueSend( pv.bletxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the BLE task */
	if (xTaskNotify( pv.ble_task, (1<<BLE_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data from the BLE Service*/
	if (xQueueReceive(pv.blerxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	res = buff[BLE_RESULT_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return res;
}

/** @brief Get BLE data
 *
 *  This function gets the data from BLE
 *  Platform service.
 *
 *
 *  @param  data[IN]  : Data to be read
 *          len [IN]  : Length of the data buffer
 *
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 */
uint8_t GetBLE(uint32_t *data, uint8_t len)
{
	uint8_t  *buff;
    uint8_t  res;
    uint32_t addr;

	/* Get the buffer from the Memory */
	buff = (uint8_t *) ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[BLE_FUNC_INDEX]     = BLE_GET;

    /* Copy the data address to the message buffer */
    addr = (uint32_t) data;
    memcpy(&buff[BLE_DATA_INDEX], &addr, sizeof(addr));

    /* Copy the data Len to the message buffer */
    memcpy(&buff[BLE_LEN_INDEX + sizeof(addr)], &len, sizeof(len));

	/* Send the data into the BLE Q */
	if (xQueueSend( pv.bletxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the BLE task */
	if (xTaskNotify( pv.ble_task, (1<<BLE_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data from the BLE Service*/
	if (xQueueReceive(pv.blerxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	res = buff[BLE_RESULT_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return res;
}
#endif	//SDK_SERVICE_BLE

#if (SDK_SERVICE_USB == PS_ENABLE)
/** @brief Send USB data
 *
 *  This function sends the USB data into the USB
 *  Platform service.
 *
 *  @param  data[IN]  : Data to be Send
 *          len [IN]  : Length of the data buffer
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 */
int8_t SetUSB(int8_t *data, uint8_t len)
{
	uint8_t *buff;
    uint8_t res;
    uint32_t addr;

	/* Get the buffer from the Memory */
	buff = (uint8_t *) ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[USB_FUNC_INDEX]     = USB_SET;

    /* Copy the data address to the message buffer */
    addr = (uint32_t) data;
    memcpy(&buff[USB_DATA_INDEX], &addr, sizeof(addr));

    /* Copy the data Len to the message buffer */
    memcpy(&buff[USB_LEN_INDEX + sizeof(addr)], &len, sizeof(len));

	/* Send the data into the USB Q */
	if (xQueueSend( pv.usbtxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the USB task */
	if (xTaskNotify( pv.usb_task, (1<<USB_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data from the USB Service*/
	if (xQueueReceive(pv.usbrxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	res = buff[USB_RESULT_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return res;
}

#endif	//SDK_SERVICE_USB

#if (SDK_SERVICE_SWTIMER == PS_ENABLE)
/** @brief Set SW Timer
 *
 *  This function sets the timer with timeout and mode value.
 *
 *
 *  @param  status    [IN] : Status : SWT_ACTIVE / SWT_INACTIVE
 *          mode      [IN] : Mode   : SWT_ONESHOT / SWT_CONTINUOUS
 *          timeoutms [IN] : Timeout in Millisecond
 *
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 */
uint8_t SetSWT(uint8_t timer_idx, uint8_t status, uint8_t mode, uint16_t timeoutms)
{
	uint8_t *buff;
    uint8_t res;

	if (!((SWT_MAX_MS >= timeoutms) && (SWT_MIN_MS <= timeoutms)))
		return FAILURE;

	/* Get the buffer from the Memory */
	buff = (uint8_t *) ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[SWT_FUNC_INDEX]    = SWT_SET;
	buff[SWT_COUNT_INDEX] 	= timer_idx;
    buff[SWT_STATUS_INDEX] 	= status;
    buff[SWT_MODE_INDEX] 	= mode;

    memcpy(&buff[SWT_TIMEMS_INDEX], &timeoutms, sizeof(timeoutms));

    /* Send the data into the SWTimer Q */
	if (xQueueSend( pv.swttxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the SWTimer task */
	if (xTaskNotify( pv.swt_task, (1<<SWT_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data from the SWTimer Service*/
	if (xQueueReceive(pv.swtrxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	res = buff[SWT_RESULT_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return res;
}

/** @brief Read Timer values
 *
 *  This function sends the read timer Command into the SWTimer
 *  Platform service.
 *
 *
 *  @param  status    [OUT] : Status : SWT_ACTIVE / SWT_INACTIVE
 *          mode      [OUT] : Mode   : SWT_ONESHOT / SWT_CONTINUOUS
 *          timeoutms [OUT] : Timeout in Millisecond
 *
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 */
uint8_t GetSWT(uint8_t *status, uint8_t *mode, uint16_t *timeoutms)
{
	uint8_t  *buff;
    uint8_t  res;

    /* Validate the pointer variables */
    if((NULL == status) || (NULL == mode) || (NULL == timeoutms))
    {
    	return FAILURE;
    }

	/* Get the buffer from the Memory */
	buff = (uint8_t *) ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[SWT_FUNC_INDEX]     = SWT_GET;

	/* Send the data into the SWTimer Q */
	if (xQueueSend( pv.swttxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the SWTimer task */
	if (xTaskNotify( pv.swt_task, (1<<SWT_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data from the SWTimer Service*/
	if (xQueueReceive(pv.swtrxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	res = buff[SWT_RESULT_INDEX];

	if (SUCCESS == res)
	{
		/* Update the data into the user variable */
		*status = buff[SWT_STATUS_INDEX];
		*mode   = buff[SWT_MODE_INDEX];
		memcpy(timeoutms, &buff[SWT_TIMEMS_INDEX], sizeof(uint16_t));
	}
	/* Release the buffer */
	ai430_memory_free(buff);

	return res;
}
#endif	//SDK_SERVICE_SWTIMER

#if (SDK_SERVICE_QSPI == PS_ENABLE)

uint8_t QSPI_Erase_Block(uint32_t BlockAddress)
{
	QSPI_CommandTypeDef		QSPI_COMMAND_ERASE_BLOCK = {
			// Instruction
			W25Q32JV_BLOCK_ERASE_64KB_INSTRUCTION,//W25Q32JV_SECTOR_ERASE_4KB_INSTRUCTION,
			// Address
			0,
			// AlternateBytes
			0,
			// AddressSize
			QSPI_ADDRESS_24_BITS,
			// AlternateBytesSize
			0,
			// DummyCycles
			0,
			// InstructionMode
			QSPI_INSTRUCTION_1_LINE,
			// AddressMode
			QSPI_ADDRESS_1_LINE,
			// AlternateByteMode
			QSPI_ALTERNATE_BYTES_NONE,
			// DataMode
			QSPI_DATA_NONE,
			// NbData
			0,
			// DdrMode
			QSPI_DDR_MODE_DISABLE,
			// DdrHoldHalfCycle
			QSPI_DDR_HHC_ANALOG_DELAY,
			// SIOOMode
			QSPI_SIOO_INST_EVERY_CMD,
	};
	QSPI_COMMAND_ERASE_BLOCK.Address = BlockAddress;

	/*	Enable write operations	*/
	if(QSPI_WriteEnable(pv.hqspi) != QSPI_OK)
	{
		return QSPI_ERROR;
	}

	/*	Send the command	*/
	if(HAL_QSPI_Command(pv.hqspi, &QSPI_COMMAND_ERASE_BLOCK, HAL_QPSI_TIMEOUT_DEFAULT_VALUE) != HAL_OK)
	{
		return QSPI_ERROR;
	}

	/*	polling mode to wait for end of erase	*/
	if(QSPI_AutoPollingMemReady(pv.hqspi, SUBSECTOR_ERASE_MAX_TIME) != QSPI_OK)
	{
		return QSPI_ERROR;
	}

	return QSPI_OK;
}

uint8_t QSPI_Write(uint8_t* pData, uint32_t WriteAddr, uint32_t Size)
{
	QSPI_CommandTypeDef		QSPI_COMMAND_WRITE = {
			// Instruction
			W25Q32JV_QUAD_INPUT_PAGE_PROGRAM_INSTRUCTION,
			// Address
			0,
			// AlternateBytes
			0,
			// AddressSize
			QSPI_ADDRESS_24_BITS,
			// AlternateBytesSize
			0,
			// DummyCycles
			0,
			// InstructionMode
			QSPI_INSTRUCTION_1_LINE,
			// AddressMode
			QSPI_ADDRESS_1_LINE,
			// AlternateByteMode
			QSPI_ALTERNATE_BYTES_NONE,
			// DataMode
			QSPI_DATA_4_LINES,
			// NbData
			0,
			// DdrMode
			QSPI_DDR_MODE_DISABLE,
			// DdrHoldHalfCycle
			QSPI_DDR_HHC_ANALOG_DELAY,
			// SIOOMode
			QSPI_SIOO_INST_EVERY_CMD,
	};

	uint32_t end_addr, current_size, current_addr;

	/*	Calculation of the size between the write address and the end of the page	*/
	current_size = PAGE_SIZE - (WriteAddr % PAGE_SIZE);

	/*	Check if the size of the data is less than the remaining place in the page	*/
	if(current_size > Size)
	{
		current_size = Size;
	}

	/*	Initialize the address variables	*/
	current_addr = WriteAddr;
	end_addr = WriteAddr + Size;

	/*	Perform the write page by page	*/
	do
	{
		QSPI_COMMAND_WRITE.Address = current_addr;
		QSPI_COMMAND_WRITE.NbData = current_size;

		/*	Enable write operations	*/
		if(QSPI_WriteEnable(pv.hqspi) != QSPI_OK)
		{
			return QSPI_ERROR;
		}

		//time delay before write instruction 5ms
		//Delayus(5000);

		if(HAL_QSPI_Command(pv.hqspi, &QSPI_COMMAND_WRITE, HAL_QPSI_TIMEOUT_DEFAULT_VALUE) != HAL_OK)
		{
			return QSPI_ERROR;
		}

		/*	Transmission of the data	*/
		if(HAL_QSPI_Transmit(pv.hqspi, pData, HAL_QPSI_TIMEOUT_DEFAULT_VALUE) != HAL_OK)
		{
			return QSPI_ERROR;
		}

		/*	automatic polling mode to wait for end of program	*/
		if(QSPI_AutoPollingMemReady(pv.hqspi, HAL_QPSI_TIMEOUT_DEFAULT_VALUE) != QSPI_OK)
		{
			return QSPI_ERROR;
		}

		/*	Update the address and size variables for next page programming	*/
		current_addr += current_size;
		pData += current_size;
		current_size = ((current_addr + PAGE_SIZE) > end_addr) ? (end_addr - current_addr) : PAGE_SIZE;
	}while(current_addr < end_addr);

	return QSPI_OK;
}

uint8_t QSPI_Read(uint8_t* pData, uint32_t ReadAddr, uint32_t Size)
{
	QSPI_CommandTypeDef		QSPI_COMMAND_READ = {
			// Instruction
			W25Q32JV_FAST_READ_QUAD_I_O_INSTRUCTION,
			// Address
			ReadAddr,
			// AlternateBytes
			0,
			// AddressSize
			QSPI_ADDRESS_24_BITS,
			// AlternateBytesSize
			0,
			// DummyCycles
			6,
			// InstructionMode
			QSPI_INSTRUCTION_1_LINE,
			// AddressMode
			QSPI_ADDRESS_4_LINES,
			// AlternateByteMode
			QSPI_ALTERNATE_BYTES_NONE,
			// DataMode
			QSPI_DATA_4_LINES,
			// NbData
			Size,
			// DdrMode
			QSPI_DDR_MODE_DISABLE,
			// DdrHoldHalfCycle
			QSPI_DDR_HHC_ANALOG_DELAY,
			// SIOOMode
			QSPI_SIOO_INST_EVERY_CMD,
	};

	if(HAL_QSPI_Command(pv.hqspi, &QSPI_COMMAND_READ, HAL_QPSI_TIMEOUT_DEFAULT_VALUE) != HAL_OK)
	{
		return QSPI_ERROR;
	}

	//(/CS driven high after a full 8-bits have been clocked
	//	Set S# timing for Read command
	MODIFY_REG(pv.hqspi->Instance->DCR, QUADSPI_DCR_CSHT, QSPI_CS_HIGH_TIME_3_CYCLE);//QSPI_CS_HIGH_TIME_3_CYCLE);

	//	Reception of the data
	if(HAL_QSPI_Receive(pv.hqspi, pData, HAL_QPSI_TIMEOUT_DEFAULT_VALUE) != HAL_OK)
	{
		return QSPI_ERROR;
	}

	//	Restore S# timing for nonRead commands
	MODIFY_REG(pv.hqspi->Instance->DCR, QUADSPI_DCR_CSHT, QSPI_CS_HIGH_TIME_6_CYCLE);//QSPI_CS_HIGH_TIME_6_CYCLE);

	return QSPI_OK;
}

#endif	//SDK_SERVICE_QSPI

#if (SDK_SERVICE_FDCAN == PS_ENABLE)

/** @brief FDCAN Configuration
 *
 *  This function sends the control data to the FDCAN
 *  through the Platform service.
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 *           2 : MEM_FAILURE
 */
int8_t ActCAN(uint8_t canid, uint8_t can_cfg, uint8_t *cfg_val)
{
	uint8_t *buff;
    uint8_t res;
    uint32_t can_id;

   	/* Get the buffer from the Memory */
   	buff = (uint8_t *) ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
   	buff[FDCAN_FUNC_INDEX] = FDCAN_ACT;
   	buff[FDCAN_FUNC_SUB_INDEX] = can_cfg;
    buff[FDCAN_FUNC_SUB_INDEX + 1] = canid;

    if(SET_FILTER_INDEX_ENABLE == can_cfg)
    {
    	/* Filter index state */
    	buff[FDCAN_FUNC_SUB_INDEX + 2] = cfg_val[0];

    	/* Filter index state value */
    	buff[FDCAN_FUNC_SUB_INDEX + 3] = cfg_val[1];

    	if(CAN_MODE_STANDARD_ID == cfg_val[2])
    	{
    		can_id = FDCAN_STANDARD_ID;
    		memcpy(&buff[FDCAN_FUNC_SUB_INDEX + 4], &can_id, sizeof(can_id));
    	}
    	else if(CAN_MODE_EXTENDED_ID == cfg_val[0])
    	{
    		can_id = FDCAN_EXTENDED_ID;
    		memcpy(&buff[FDCAN_FUNC_SUB_INDEX + 4], &can_id, sizeof(can_id));
    	}
    }
    else if(SET_FDCAN_RESET == can_cfg)
    {
    	buff[FDCAN_FUNC_SUB_INDEX + 2] = cfg_val[0];
    }
    else if(SET_FDCAN_DRIVER_RESET == can_cfg)
    {
    	buff[FDCAN_FUNC_SUB_INDEX + 2] = cfg_val[0];
    }
    else if(SET_FDCAN_BAUDRATE == can_cfg)
    {
    	buff[FDCAN_FUNC_SUB_INDEX + 2] = cfg_val[0];
    }

	/* Send the data into the FDCAN Q */
	if (xQueueSend( pv.fdtxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the FDCAN task */
	if (xTaskNotify( pv.fdcan_task, (1<<FDCAN_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data from the FDCAN Service*/
	if (xQueueReceive(pv.fdrxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	res = buff[FDCAN_RESULT_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return res;
}

/** @brief Send FDCAN data
 *
 *  This function sends the FDCAN data into the FDCAN
 *  Platform service.
 *
 *  @param  data[IN]  : Data to be Send
 *          len [IN]  : Length of the data buffer
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 */
int8_t SetCAN(uint8_t canid, uint8_t *data, uint8_t len)
{
	uint8_t *buff;
    uint8_t res;
    uint32_t addr;

	/* Get the buffer from the Memory */
	buff = (uint8_t *) ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

   	if(canid == FDCAN_1)
   	{
   		/* Construct the buffer */
   		buff[FDCAN_FUNC_INDEX] = FDCAN1_SET;
   	}
   	else if(canid == FDCAN_2)
   	{
   		buff[FDCAN_FUNC_INDEX] = FDCAN2_SET;
   	}

    /* Copy the data address to the message buffer */
    addr = (uint32_t) data;
    memcpy(&buff[FDCAN_DATA_INDEX], &addr, sizeof(addr));

    /* Copy the data Len to the message buffer */
    memcpy(&buff[FDCAN_LEN_INDEX + sizeof(addr)], &len, sizeof(len));

	/* Send the data into the FDCAN Q */
	if (xQueueSend( pv.fdtxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the FDCAN task */
	if (xTaskNotify( pv.fdcan_task, (1<<FDCAN_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data from the FDCAN Service*/
	if (xQueueReceive(pv.fdrxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	res = buff[FDCAN_RESULT_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return res;
}

#endif	//SDK_SERVICE_FDCAN

#if (SDK_SERVICE_CAMERA == PS_ENABLE)
/** @brief Start/Stop the Camera Streaming
 *
 *  This function sends the camera streaming information to the
 *  Platform service.
 *
 *  @param  mode[IN]  : streaming mode
 *          x_org[IN] : X origin value
 *          y_org[IN] : Y origin value
 *          width[IN] : Width size
 *          height[IN]: Height size
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 */
uint8_t SetCam(uint8_t mode, uint8_t *buf)
{
	uint8_t *buff;
    uint8_t res;

	/* Get the buffer from the Memory */
	buff = (uint8_t *) ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[CAMERA_FUNC_INDEX] = mode;

    /* Copy the data to the message buffer */
	memcpy(&buff[CAMERA_X_ORG_INDEX+1], &buf[CAMERA_X_ORG_INDEX], sizeof(uint16_t));
	memcpy(&buff[CAMERA_Y_ORG_INDEX+1], &buf[CAMERA_Y_ORG_INDEX], sizeof(uint16_t));
	memcpy(&buff[CAMERA_WIDTH_INDEX+1], &buf[CAMERA_WIDTH_INDEX], sizeof(uint16_t));
	memcpy(&buff[CAMERA_HEIGHT_INDEX+1], &buf[CAMERA_HEIGHT_INDEX], sizeof(uint16_t));
	memcpy(&buff[DISPLAY_X_ORG_INDEX+1], &buf[DISPLAY_X_ORG_INDEX], sizeof(uint16_t));
	memcpy(&buff[DISPLAY_Y_ORG_INDEX+1], &buf[DISPLAY_Y_ORG_INDEX], sizeof(uint16_t));
	memcpy(&buff[DISPLAY_FLIP+1], &buf[DISPLAY_FLIP], sizeof(uint8_t));
	memcpy(&buff[STREAM_MODE+1], &buf[STREAM_MODE], sizeof(uint8_t));

	/* Send the data into the Camera Q */
	if (xQueueSend( pv.cameratxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the Camera task */
	if (xTaskNotify( pv.camera_task, (1<<CAMERA_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data from the Camera Service*/
	if (xQueueReceive(pv.camerarxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	res = buff[CAMERA_RESULT_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return res;
}

#endif	//SDK_SERVICE_CAMERA

#if (SDK_SERVICE_J1939 == PS_ENABLE)
/** @brief Sends j1939 data
 *
 *  This function sends the j1939 control data to the
 *  Platform service.
 *
 *
 *  @param  j1939_fun[IN]   : CAN0_DM2/CAN1_DM2/
							  CAN0_DM4/CAN1_DM4/
							  CAN0_DM11/CAN1_DM11
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 */
uint8_t Setj1939(uint8_t j1939_fun, uint8_t *data)
{
	uint8_t *buff;
    uint8_t res;
    uint32_t addr;

	if ((CAN0_DM1 != j1939_fun) && (CAN1_DM1 != j1939_fun) &&
			(CAN0_DM2 != j1939_fun) && (CAN1_DM2 != j1939_fun) &&
			(CAN0_DM3 != j1939_fun) && (CAN1_DM3 != j1939_fun))
		return FAILURE;

	/* Get the buffer from the Memory */
	buff = (uint8_t *) ai430_memory_alloc();

   	if (NULL == buff)
		return MEM_FAILURE;

	/* Construct the buffer */
	buff[J1939_FUNC_INDEX] 	 = SET_J1939;
	if(CAN0_DM1 == j1939_fun)
	{
		buff[J1939_SUB_FUNC_INDEX] = SEND_CAN0_DM1_MSG;
	}
	else
	if(CAN1_DM1 == j1939_fun)
	{
		buff[J1939_SUB_FUNC_INDEX] = SEND_CAN1_DM1_MSG;
	}
	else
	if(CAN0_DM2 == j1939_fun)
	{
		buff[J1939_SUB_FUNC_INDEX] = SEND_CAN0_DM2_MSG;
	}
	else
	if(CAN1_DM2 == j1939_fun)
	{
		buff[J1939_SUB_FUNC_INDEX] = SEND_CAN1_DM2_MSG;
	}
		else
	if(CAN0_DM3 == j1939_fun)
	{
		buff[J1939_SUB_FUNC_INDEX] = SEND_CAN0_DM3_MSG;
	}
	else
	if(CAN1_DM3 == j1939_fun)
	{
		buff[J1939_SUB_FUNC_INDEX] = SEND_CAN1_DM3_MSG;
	}
	else
	if(CAN0_D_ADD == j1939_fun)
	{
		buff[J1939_SUB_FUNC_INDEX] = CAN0_DEST_ADDRESS;

		addr = (uint32_t) data;
	    memcpy(&buff[J1939_SUB_FUNC_INDEX+1], &addr, sizeof(addr));
	}
	else
	if(CAN1_D_ADD == j1939_fun)
	{
		buff[J1939_SUB_FUNC_INDEX] = CAN1_DEST_ADDRESS;

		addr = (uint32_t) data;
	    memcpy(&buff[J1939_SUB_FUNC_INDEX+1], &addr, sizeof(addr));
	}

	/* Send the data into the J1939 Q */
	if (xQueueSend( pv.jctrltxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the J1939 task to update the j1939 data */
	if (xTaskNotify( pv.j1939_ctrltask, (1<<J1939_MSG_EVENT), eSetBits ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Wait for receiving the return data */
    /* Send the data to the J1939 platform service layer */
	if (xQueueReceive(pv.jctrlrxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	res = buff[J1939_RESULT_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return res;
}
#endif	//SDK_SERVICE_J1939

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
