/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : OPT3002.C
   Author             : Francisco Orozco
   Version            : V0.0.0.1
   Origination Date   : 28/01/2022
   Description        : OPT3002 Driver C Source File.
   Environment        : IAR 8.40
   Processor:         : ST STM32F746IG

-----------------------------------------------------------------------------
   Description:

		General File Description
-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.0             11/22/2019           Francisco Orozco

   Notes:              - Initial Version
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             28/01/2022           Zaid

   Notes:              - Second Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/
#include "main.h"
#include "../inc/OPT3002.h"
#include "../../../Core/Maximatecc/Inc/Common.h"


/** @brief Light Sensor Initialization
 *
 *  This function initializes the Light Sensor device
 *
 *  @param  convtime[IN] : Conversion Time
 *          mode [IN]    : Mode of the Light Sensor operation (Single / Continuous)
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 *
 */
opt_drv_t SOPTdriver_Init(I2C_HandleTypeDef *hi2c, uint8_t convtime, uint8_t mode)
{
	uint8_t i2cBuf[8];
	opt_drv_t err;

	/* Validate the Conversion time value */
	if ( (CONF_REG_CONVERSION_TIME_100 != convtime) &&
			(CONF_REG_CONVERSION_TIME_800 != convtime) )
		return OPTERR_ERROR_OK;

	/* Validate the Mode value */
	if ( (CONF_REG_SINGLE_MODE_CONV != mode) &&
			(CONF_REG_CONTINUOUS_MODE_CONV != mode) && (CONF_REG_SHUTDOWN != mode) )
		return OPTERR_ERROR_OK;

	if (HAL_I2C_IsDeviceReady(hi2c,Addr_OPT3002,1,10)!=HAL_OK)
		return OPTERR_NOT_READY;

	/* Configuration */
	i2cBuf[0]=0x01; // Configuration Register Address
	/* Configuration Field Value */
	i2cBuf[1] = (AUTOMATIC_FULL_SCALE << CONV_TIME_BIT_POSITION) |
				(convtime << MODE_BIT_POSITION) | (mode << LS_CFG_BIT_POSITION);
	 // bit 15-12 = 0xC Range number field (read or write) C=Automatic full scale,
	 // bit 11    = 1 Conversion time field (read or write) 1=set to 800ms
	 // bit 10-9  = 10 Mode of conversion operation field (read or write).
	 // 			   10=Single shoot,  when conversion is complete, is automatically set to 00b and the device is shut down.
	 // bit 8     = 0 Overflow flag field (read-only).
	i2cBuf[2]=0x00; // bit 7     = 0 Conversion ready field (read-only).
	 // bit 6     = 0 Flag high field (read-only).
	 // bit 5     = 0 Flag low field (read-only).
	 // bit 4     = 0 Latch field (read or write). 0 = The device functions in transparent hysteresis-style
	 // bit 3     = 0 Polarity field (read or write). 1= Operation of the INT pin is inverted,
	 // 			   where the INT pin reports active high, 0 = The INT pin reports active low,
	 // 			   pulling the pin low upon an interrupt event
	 // bit 2     = 0 Mask exponent field (read or write). The mask exponent field forces the result
	 // 			   register exponent field (register 00h, bits E[3:0])
	 // bit 1-0   = 00 Fault count field (read or write) to trigger the interrupt reporting mechanism
	 // 00 		 = One fault count (default) 01 = Two fault counts 10 = Four fault counts 11 = Eight fault counts

	if (HAL_I2C_Master_Transmit(hi2c,Addr_OPT3002,i2cBuf,3,10) != HAL_OK)
		return OPTERR_SET_OPERATION;

	err = SOPTdriver_LoLimit(hi2c, 0);

	if (err!=OPTERR_ERROR_OK)
		return err;

	err= SOPTdriver_HiLimit(hi2c, 0xFFFF);

	if (err!=OPTERR_ERROR_OK)
		return err;

	return OPTERR_ERROR_OK;
}

/** @brief Get Configuration val
 *
 *  This function gets the conversion time value from
 *  OPT3002 configuration register
 *
 *  @param  convtime[OUT] : return the conversion time
 *
 *  @return Conversion time data
 *          OPTERR_NOT_READY
 *          OPTERR_SET_ADD
 *          OPTERR_RESULT
 *          OPTERR_ERROR_OK
 *
 */
uint8_t SOPTdriver_GetCovntime(I2C_HandleTypeDef *hi2c, uint8_t *convtime)
{
	uint8_t i2cBuf[8];

	if (HAL_I2C_IsDeviceReady(hi2c,Addr_OPT3002,1,10)!=HAL_OK)
		return OPTERR_NOT_READY;

	/*	Result Address */
	i2cBuf[0]=0x01;

	/* EX3 EX2 EX1 EX0 R11 R10 R9 R8 */
	i2cBuf[1]=0;

	/*	R7 R6 R5 R4 R3 R2 R1 R0	*/
	i2cBuf[2]=0;

	if (HAL_I2C_Master_Transmit(hi2c,Addr_OPT3002,i2cBuf,1,50)!=HAL_OK)
		return OPTERR_SET_ADD;

	if (HAL_I2C_Master_Receive(hi2c,Addr_OPT3002,&i2cBuf[1],2,50)!=HAL_OK)
		return OPTERR_RESULT;

	/* Get the Value Conversion time from the configuration reg*/
	*convtime = ((i2cBuf[1] & OPT_CONF_CONVERSION_TIME_MASK) >>
			OPT_CONF_CONVERSION_TIME_BIT_POSITION);

	return OPTERR_ERROR_OK;

}


/** @brief Get Mode val
 *
 *  This function gets the Mode value from
 *  OPT3002 configuration register
 *
 *  @param  mode[OUT] : return the mode value
 *
 *  @return OPTERR_NOT_READY
 *          OPTERR_SET_ADD
 *          OPTERR_RESULT
 *          OPTERR_ERROR_OK
 *
 */
uint8_t SOPTdriver_GetMode(I2C_HandleTypeDef *hi2c, uint8_t *mode)
{
	uint8_t i2cBuf[8];

	if (HAL_I2C_IsDeviceReady(hi2c,Addr_OPT3002,1,10)!=HAL_OK)
		return OPTERR_NOT_READY;

	/*	Result Address	*/
	i2cBuf[0]=0x01;

	/*	EX3 EX2 EX1 EX0 R11 R10 R9 R8 */
	i2cBuf[1]=0;

	/*	R7 R6 R5 R4 R3 R2 R1 R0	*/
	i2cBuf[2]=0;

	if (HAL_I2C_Master_Transmit(hi2c,Addr_OPT3002,i2cBuf,1,50)!=HAL_OK)
		return OPTERR_SET_ADD;

	if (HAL_I2C_Master_Receive(hi2c,Addr_OPT3002,&i2cBuf[1],2,50)!=HAL_OK)
		return OPTERR_RESULT;

	/* Get the Value Mode from the configuration reg*/
	*mode = ((i2cBuf[1] & OPT_CONF_MODE_MASK) >>
			OPT_CONF_MODE_BIT_POSITION);

	return OPTERR_ERROR_OK;
}

/** @brief Shutdown device
 *
 *  This function puts the device to shutdown
 *
 *  @param  none
 *
 *  @return Mode value
 *          OPTERR_NOT_READY
 *          OPTERR_SET_ADD
 *          OPTERR_RESULT
 *          OPTERR_ERROR_OK
 *
 */
uint8_t SOPTdriver_Shutdown(I2C_HandleTypeDef *hi2c)
{
	uint8_t i2cBuf[8];
	uint8_t val;

	/* Read the configuration register */
	if (HAL_I2C_IsDeviceReady(hi2c,Addr_OPT3002,1,10)!=HAL_OK)
		return OPTERR_NOT_READY;

	/*	Result Address */
	i2cBuf[0]=0x01;

	/*	EX3 EX2 EX1 EX0 R11 R10 R9 R8 */
	i2cBuf[1]=0;

	/*	R7 R6 R5 R4 R3 R2 R1 R0 */
	i2cBuf[2]=0;

	if (HAL_I2C_Master_Transmit(hi2c,Addr_OPT3002,i2cBuf,1,50)!=HAL_OK)
		return OPTERR_SET_ADD;

	if (HAL_I2C_Master_Receive(hi2c,Addr_OPT3002,&i2cBuf[1],2,50)!=HAL_OK)
		return OPTERR_RESULT;

	val = i2cBuf[1] & (~OPT_CONF_MODE_MASK);

	if (HAL_I2C_IsDeviceReady(hi2c,Addr_OPT3002,1,10)!=HAL_OK)
		return OPTERR_NOT_READY;

	/*	Configuration */
	i2cBuf[0]=0x01;
	i2cBuf[1]=val;

	if (HAL_I2C_Master_Transmit(hi2c,Addr_OPT3002,i2cBuf,3,10)!=HAL_OK)
		return OPTERR_SET_OPERATION;

	return OPTERR_ERROR_OK;

}


/*******************************************************************************
* Function Name  : SOPTdriver_DeInit
* Description    : Initialize Light Sensor Driver
* Input          : None
* Output         : None
* Return         : Current operation error state.
*******************************************************************************/
opt_drv_t SOPTdriver_DeInit(I2C_HandleTypeDef *hi2c)
{
	uint8_t i2cBuf[8];
	opt_drv_t err;
	if (HAL_I2C_IsDeviceReady(hi2c,Addr_OPT3002,1,10)!=HAL_OK)
		return OPTERR_NOT_READY;

	/* Configuration
	 * Configuration Register Address
	 * bit 15-12 =0xC Range number field (read or write) C=Automatic full scale (default)
	 * bit 11    =1 Conversion time field (read or write) 1=set to 800ms
	 * bit 10-9  =00 Mode of conversion operation field (read or write). 00=Shutdown (default),
	 * 				when conversion is complete, is automatically set to 00b and the device is shut down.
	 * bit 8     =0 Overflow flag field (read-only).
	 * bit 7     =0 Conversion ready field (read-only).
	 * bit 6     =0 Flag high field (read-only).
	 * bit 5     =0 Flag low field (read-only).
	 * bit 4     =0 Latch field (read or write). 0 = The device functions in transparent hysteresis-style
	 * bit 3     =0 Polarity field (read or write). 1= Operation of the INT pin is inverted,
	 * 				where the INT pin reports active high, 0 = The INT pin reports active low,
	 * 				pulling the pin low upon an interrupt event
	 * bit 2     =0 Mask exponent field (read or write). The mask exponent field forces the result register
	 * 				exponent field (register 00h, bits E[3:0])
	 * bit 1-0   =00 Fault count field (read or write) to trigger the interrupt reporting mechanism
	 * 00 = One fault count (default) 01 = Two fault counts 10 = Four fault counts 11 = Eight fault counts
	 * */

	i2cBuf[0]=0x01;
	i2cBuf[1]=0xC8;
	i2cBuf[2]=0;

	if (HAL_I2C_Master_Transmit(hi2c,Addr_OPT3002,i2cBuf,3,10)!=HAL_OK)
		return OPTERR_SET_OPERATION;

	err= SOPTdriver_LoLimit(hi2c, 0);

	if (err!=OPTERR_ERROR_OK)
		return err;

	err= SOPTdriver_HiLimit(hi2c, 0xFFFF);

	if (err!=OPTERR_ERROR_OK)
		return err;

	return OPTERR_ERROR_OK;

}

/*******************************************************************************
* Function Name  : SOPTdriver_LoLimit
* Description    : Set LoLimit TH
* Input          : uint16_t, low limit value
* Output         : None
* Return         : Current operation error state.
*******************************************************************************/  
opt_drv_t SOPTdriver_LoLimit(I2C_HandleTypeDef *hi2c, uint16_t lowlimit)
{
	uint8_t i2cBuf[8];

	/*	Low-Limit Register Address */
	i2cBuf[0] = 0x2;
	i2cBuf[1] = (lowlimit>>8) & 0xFF;
	i2cBuf[2] = lowlimit&0xFF;

	if (HAL_I2C_Master_Transmit(hi2c,Addr_OPT3002,i2cBuf,3,10)!=HAL_OK)
		return OPTERR_SET_LL;

	return OPTERR_ERROR_OK;

}

/*******************************************************************************
* Function Name  : SOPTdriver_HiLimit
* Description    : Set HiLimit TH
* Input          : uint16_t, low limit value
* Output         : None
* Return         : Current operation error state.
*******************************************************************************/  
opt_drv_t SOPTdriver_HiLimit(I2C_HandleTypeDef *hi2c, uint16_t highlimit)
{
	uint8_t i2cBuf[8];

	/*	High-Limit Register Address */
	i2cBuf[0] = 0x3;
	i2cBuf[1] = (highlimit>>8) & 0xFF;
	i2cBuf[2] = highlimit & 0xFF;

	if (HAL_I2C_Master_Transmit(hi2c,Addr_OPT3002,i2cBuf,3,10)!=HAL_OK)
		return OPTERR_SET_HL;

	return OPTERR_ERROR_OK;

}

/*******************************************************************************
* Function Name  : SOPTdriver_Result
* Description    : Read  Optical Power nW/cm2
* Input          : None
* Output         : None
* Return         : Decoding the Result Register into Optical Power.
*******************************************************************************/  
opt_drv_t SOPTdriver_Result(I2C_HandleTypeDef *hi2c, uint32_t *val)
{
	uint16_t x;
	uint8_t  y;
	uint8_t i2cBuf[8];

	memset(&i2cBuf[0], 0x00, sizeof(i2cBuf));
	/*	Result Address */
	i2cBuf[0]=0x01;

	/*	EX3 EX2 EX1 EX0 R11 R10 R9 R8 */
	i2cBuf[1]=0x00;

	/*	R7 R6 R5 R4 R3 R2 R1 R0 */
	i2cBuf[2]=0;

	if (HAL_I2C_Master_Transmit(hi2c,Addr_OPT3002,&i2cBuf[0],1,50)!=HAL_OK)
		return OPTERR_SET_ADD;

	if (HAL_I2C_Master_Receive(hi2c,Addr_OPT3002,&i2cBuf[1],2,50)!=HAL_OK)
		return OPTERR_RESULT;


	/*	Result Address */
	i2cBuf[0]=0x00;

	/*	EX3 EX2 EX1 EX0 R11 R10 R9 R8 */
	i2cBuf[1]=0;

	/*	R7 R6 R5 R4 R3 R2 R1 R0 */
	i2cBuf[2]=0;

	if (HAL_I2C_Master_Transmit(hi2c,Addr_OPT3002,&i2cBuf[0],1,50)!=HAL_OK)
		return OPTERR_SET_ADD;

	if (HAL_I2C_Master_Receive(hi2c,Addr_OPT3002,&i2cBuf[1],2,50)!=HAL_OK)
		return OPTERR_RESULT;

	x = i2cBuf[1]&0xF;

	/*	Get Fractional Result */
	x = x<<8|i2cBuf[2];

	/*	Get Exponent */
	y = i2cBuf[1]>>4;
	*val = pow(2,y)*x*1.2;

	return OPTERR_ERROR_OK;

}

/*******************************************************************************
* Function Name  : SOPTdriver_RawResult
* Description    : Read  Optical Power nW/cm2 (Raw Registers)
* Input          : None
* Output         : Array Raw Data  
* Return         : Current operation error state.
*******************************************************************************/  
opt_drv_t SOPTdriver_RawResult(I2C_HandleTypeDef *hi2c, uint8_t *rsp_data)
{
	uint8_t i2cBuf[8];

	/*	Result Address */
	i2cBuf[0]=0x00;

	/*	EX3 EX2 EX1 EX0 R11 R10 R9 R8 */
	i2cBuf[1]=0;

	/*	R7 R6 R5 R4 R3 R2 R1 R0 */
	i2cBuf[2]=0;

	if (HAL_I2C_Master_Transmit(hi2c,Addr_OPT3002,i2cBuf,1,10)!=HAL_OK)
		return OPTERR_SET_ADD;

	if (HAL_I2C_Master_Receive(hi2c,Addr_OPT3002,&i2cBuf[1],2,10)!=HAL_OK)
		return OPTERR_RESULT;

	/*	Fractional Result LS */
	rsp_data[0] = i2cBuf[2];

	/*	Fractional Result HS */
	rsp_data[1] = i2cBuf[1]&0xF;

	/*	Get Exponent */
    rsp_data[2] = i2cBuf[1]>>4;
    rsp_data[3] = 0 ;

    return OPTERR_ERROR_OK;

}

/*******************************************************************************
* Function Name  : SOPTdriver_Manufacturer_ID
* Description    : Manufacturer ID Register Field Descriptions
* Input          : None
* Output         : None
* Return         : The manufacturer ID reads 5449h. In ASCII code, this register reads TI.
*******************************************************************************/  
uint16_t SOPTdriver_Manufacturer_ID(I2C_HandleTypeDef *hi2c)
{
	uint8_t i2cBuf[8] = {0};
	uint16_t val;

	/*	Set to 0 */
	i2cBuf[2]=0;

	/*	Set to 0 */
	i2cBuf[1]=0;

	/*	Manufacturer ID Address */
	i2cBuf[0]=0x7E;

	if (HAL_I2C_Master_Transmit(hi2c,Addr_OPT3002,i2cBuf,1,100)!=HAL_OK)
		return OPTERR_SET_ADD;

	if (HAL_I2C_Master_Receive(hi2c,Addr_OPT3002,&i2cBuf[1],2,100)!=HAL_OK)
		return OPTERR_MANUF_ID;

	val = ((i2cBuf[1]<<8) | i2cBuf[2]);
	/*	0x5449 = ASCII TI */
	return val;

}

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
