/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Warning_light.c
   Author             : Rajesh
   Version            : V0.0.0.1
   Origination Date   : 21/01/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the Warning Light HAL driver functions

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             21/01/2022           Rajesh

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../../../Core/Maximatecc/Inc/Common.h"
/* USER CODE END Includes */

#if (SDK_SERVICE_WARNING_LIGHT == PS_ENABLE)
/* USER CODE BEGIN Global Variables */
/* USER CODE END Global Variables */


/** @brief Warning light Initialization
 *
 *  This function initialize the Warning Light device
 *
 *  @param  freq [IN] : 0 - 3kHz
 *                    : 1 - 22kHz
 *                
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */
int8_t Warning_Light_Init(I2C_HandleTypeDef *hi2c, uint8_t freq)
{
	int8_t ret;
	uint8_t val;

	/* Check for the frequency */
	/* if it is invalid, choose the default as 3kHz */
	if ((kHz3_FREQ != freq) && (kHz22_FREQ != freq))
		freq = kHz3_FREQ;

    HAL_GPIO_WritePin(HW_WLSTB_GPIO_Port, HW_WLSTB_Pin, GPIO_PIN_SET);

	/* Enable the normal mode operation in the Shutdown reg */
	val = SOFTWARE_NORMAL_MODE;
    ret = HAL_I2C_Mem_Write(hi2c, IS31FL3235_IIC_ADDRESS,
			IS31FL3235_REG_WORK, sizeof(uint8_t), &val, sizeof(val), I2C_TIMEOUT);

    /* Enable the Warning Light */
	val = GC_NORMAL_OPERATION;
	ret = HAL_I2C_Mem_Write(hi2c, IS31FL3235_IIC_ADDRESS, IS31FL3235_REG_LED_SYNC,
			sizeof(uint8_t), &val,sizeof(val),I2C_TIMEOUT);
	
    if (HAL_OK != ret)
		return PS_FAILURE;

    /* Set the Warning Light Freqency */
	ret = HAL_I2C_Mem_Write(hi2c, IS31FL3235_IIC_ADDRESS,
	           IS31FL3235A_REG_FREQ, sizeof(uint8_t), &freq, sizeof(freq), I2C_TIMEOUT);
	
    if (HAL_OK != ret)
		return PS_FAILURE;

    Turn_Off_Warning_Light(hi2c);
	return PS_SUCCESS;
}


/** @brief Warning light On
 *
 *  This function is used for turn on warning light.
 *
 *  @param  led_num [IN]: LED (Channel) number 
 * 
 *          current [IN]: 0 : Max current
 *                        1 : 1/Max current
 *                        2 : 2/Max current
 *                        3 : 3/Max current
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */
int8_t Warning_Light_On(I2C_HandleTypeDef *hi2c, uint8_t led_num, uint8_t current)
{
	uint8_t val;
	HAL_StatusTypeDef ret;

    /* Check for the LED number */
    if (!((WL_MAX_LED_NUMBER > led_num) && (WL_MIN_LED_NUMBER <= led_num)))
	     return PS_FAILURE;
	
    /* Check for the current value */
	if (!((WL_MAX_CURRENT >= current) && (WL_MIN_CURRENT <= current)))
		current = 0;

    led_num = IS31FL3235_REG_CTRL_OUT1 + led_num ;
	
	val = (current << IS31FL3235_REG_CTRL_OUT_CURR_BIT) | WL_LED_ON;
	
    /* Turn on warning light.*/
	ret = HAL_I2C_Mem_Write(hi2c, IS31FL3235_IIC_ADDRESS, led_num,
			        sizeof(led_num),&val,sizeof(val),I2C_TIMEOUT);
    if (HAL_OK != ret)
		return PS_FAILURE;

    /* Update into the register */
	val = 0x00;
	ret = HAL_I2C_Mem_Write(hi2c, IS31FL3235_IIC_ADDRESS, IS31FL3235_REG_DATA_UPDATE,
			sizeof(uint8_t), &val,sizeof(val),I2C_TIMEOUT);
	
    if (HAL_OK != ret)
		return PS_FAILURE;

	return PS_SUCCESS;
}


/** @brief Turn OFF WL
 *
 *  This function is used to turn off all the warning lights.
 *
 *  @param  None
 *
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */
void Turn_Off_Warning_Light(I2C_HandleTypeDef *hi2c)
{
	uint8_t  index;
	uint8_t  res = PS_SUCCESS;

	/* Turn off all the warning lights */
	for (index=0; index < WL_MAX_LED_NUMBER; index++)
	{
		/* Turn off the Warning Light */
		res = Warning_Light_Off(hi2c, index);

		if (HAL_OK != res)
			res = PS_FAILURE;
	}
}

/** @brief Warning light OFF
 *
 *  This function is used for turn off warning light.
 *
 *  @param  led_num [IN]: LED (Channel) number 
 * 
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */
int8_t Warning_Light_Off(I2C_HandleTypeDef *hi2c, uint8_t led_num)
{
	uint8_t           val;
	HAL_StatusTypeDef ret;

    /* Check for the LED number */
    if (!((WL_MAX_LED_NUMBER > led_num) && (WL_MIN_LED_NUMBER <= led_num)))
	     return PS_FAILURE;

    led_num = IS31FL3235_REG_CTRL_OUT1 + led_num ;
	
	val = WL_LED_OFF;
	
    /* Turn off warning light */
	ret = HAL_I2C_Mem_Write(hi2c, IS31FL3235_IIC_ADDRESS, led_num,
	                          sizeof(led_num),&val,sizeof(val),I2C_TIMEOUT);
    if (HAL_OK != ret)
		return PS_FAILURE;

    /* Update into the register */
    led_num = 0x00;
	ret = HAL_I2C_Mem_Write(hi2c, IS31FL3235_IIC_ADDRESS, IS31FL3235_REG_DATA_UPDATE,
			sizeof(uint8_t), &led_num,sizeof(led_num),I2C_TIMEOUT);
	
    if (HAL_OK != ret)
		return PS_FAILURE;

	return PS_SUCCESS;
}


/** @brief Warning light brightness
 *
 *  This function is used for setting the brightness in LED.
 *
 *  @param  led_num [IN]: LED (Channel) number 
 *          intenval[IN]: Brightness value
 * 
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */
int8_t Set_Warning_Light_Intensity(I2C_HandleTypeDef *hi2c, uint8_t led_num, uint8_t intenval)
{
	HAL_StatusTypeDef ret;
	uint8_t duty_cycle = 0;

    /* Check for the LED number */
    if (!((WL_MAX_LED_NUMBER > led_num) && (WL_MIN_LED_NUMBER <= led_num)))
	     return PS_FAILURE;

    /* Check for the LED number */
    if (!((WL_MAX_INTENSITY >= intenval) && (WL_MIN_INTENSITY <= intenval)))
	     return PS_FAILURE;

    led_num = IS31FL3235_REG_PWM_OUT1 + led_num ;

    duty_cycle = (intenval * 2.55);

    /* Turn off warning light */
	ret = HAL_I2C_Mem_Write(hi2c, IS31FL3235_IIC_ADDRESS, led_num,
	                     sizeof(led_num), (uint8_t *)&duty_cycle, sizeof(duty_cycle), I2C_TIMEOUT);
						 
    if (HAL_OK != ret)
		return PS_FAILURE;

    /* Update into the register */
    led_num = 0x00;
	ret = HAL_I2C_Mem_Write(hi2c, IS31FL3235_IIC_ADDRESS, IS31FL3235_REG_DATA_UPDATE,
			sizeof(uint8_t), &led_num,sizeof(led_num),I2C_TIMEOUT);
	
    if (HAL_OK != ret)
		return PS_FAILURE;


	return PS_SUCCESS;
}

/** @brief Shutdown the Warning light
 *
 *  This function is used to shutdown the Warning light device
 *
 *  @param  none
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */
int8_t Shutdown_Warning(I2C_HandleTypeDef *hi2c)
{
	int8_t ret;
	uint8_t val;

	/* Software shutdown mode operation in the Shutdown reg */
	val = SOFTWARE_SHUTDOWN_MODE;
    ret = HAL_I2C_Mem_Write(hi2c, IS31FL3235_IIC_ADDRESS,
			IS31FL3235_REG_WORK, sizeof(uint8_t), &val, sizeof(val), I2C_TIMEOUT);

	/* Shutdown all LEDs */
	val = GC_SHUTDOWN_ALL_LEDS;
	ret = HAL_I2C_Mem_Write(hi2c, IS31FL3235_IIC_ADDRESS, IS31FL3235_REG_LED_SYNC,
			sizeof(uint8_t), &val,sizeof(val),I2C_TIMEOUT);

    if (HAL_OK != ret)
		return PS_FAILURE;

    /* Put the chip into the Hardware Shutdown */
    HAL_GPIO_WritePin(HW_WLSTB_GPIO_Port, HW_WLSTB_Pin, GPIO_PIN_RESET);

    return PS_SUCCESS;
}

#endif	//SDK_SERVICE_WARNING_LIGHT

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
